"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var T = require("fp-ts/lib/Task");
var O = require("fp-ts/lib/Option");
var pipeable_1 = require("fp-ts/lib/pipeable");
var _1 = require(".");
/**
 * Apply policy and delay by its amount if it results in a retry.
 * Returns updated status.
 *
 * @since 0.1.0
 */
function applyAndDelay(policy, status) {
    var newStatus = _1.applyPolicy(policy, status);
    return pipeable_1.pipe(newStatus.previousDelay, O.fold(function () { return T.task.of(newStatus); }, function (millis) { return T.delay(millis)(T.task.of(newStatus)); }));
}
exports.applyAndDelay = applyAndDelay;
/**
 * Retry combinator for actions that don't raise exceptions, but
 * signal in their type the outcome has failed. Examples are the
 * `Option`, `Either` and `EitherT` monads.
 *
 * @since 0.1.0
 */
function retrying(policy, action, check) {
    var go = function (status) {
        return pipeable_1.pipe(status, action, T.chain(function (a) {
            if (check(a)) {
                return pipeable_1.pipe(applyAndDelay(policy, status), T.chain(function (status) {
                    return pipeable_1.pipe(status.previousDelay, O.fold(function () { return T.task.of(a); }, function () { return go(status); }));
                }));
            }
            else {
                return T.task.of(a);
            }
        }));
    };
    return go(_1.defaultRetryStatus);
}
exports.retrying = retrying;
//# sourceMappingURL=Task.js.map